float fogify(float x, float w) {
	return w / (x * x + w);
}

vec2 rotate(vec2 P, float Ang) {
    float cosT = cos(Ang);
    float sinT = sin(Ang);
    return vec2(
        P.x*cosT-P.y*sinT,
        P.y*cosT+P.x*sinT
    );
}

vec3 round_sun(float Dist) {
    const vec3 SUN_COLOR = vec3(5, 3.5, 0.8);
    const vec3 MOON_COLOR = vec3(1, 1.5, 2.5);
    float What = sunriseStrength+sunsetStrength;
    vec3 Color = SUN_COLOR * (1-smoothstep(0., 0.1, Dist)) * (dayStrength+What);
    Color += MOON_COLOR * (smoothstep(1.999, 2., Dist)) * (nightStrength+What);
    return Color;
}

vec3 get_sun_glare(float Dist) {
    float Visibility = sunsetStrength+sunriseStrength;
    
    const vec3 SUN_GLARE = to_linear(vec3(0.8, 0.45, 0.0));

    return SUN_GLARE*clamp(1-Dist/2, 0, 1)*Visibility;
}

vec3 get_cloud_lighting(vec2 Coords) {
	const float Offset = 0.2;
	float D0 = fbm_clouds(vec2(Coords.x+Offset, Coords.y), CLOUD_LIGHTING_QUALITY);
	float D1 = fbm_clouds(vec2(Coords.x-Offset, Coords.y), CLOUD_LIGHTING_QUALITY);
	float D2 = fbm_clouds(vec2(Coords.x, Coords.y+Offset), CLOUD_LIGHTING_QUALITY);
    float D3 = fbm_clouds(vec2(Coords.x, Coords.y-Offset), CLOUD_LIGHTING_QUALITY);
    float A = (D0-D1)/Offset;
    float B = (D2-D3)/Offset;
	return normalize(vec3(A, 1-(A*A+B*B), B));
}

vec3 get_clouds(vec3 CloudPos, vec3 SkyColor, vec3 SunGlare) {
    const float ACTUAL_CLOUD_SPEED = CLOUD_SPEED/100.;
    float Animation = float(frameTimeCounter)*ACTUAL_CLOUD_SPEED;

    CloudPos.xz += cameraPosition.xz/512;
    CloudPos.xz = CloudPos.xz*CLOUD_SIZE+Animation;

    float Noise = fbm_clouds(CloudPos.xz, CLOUD_QUALITY);
    Noise = pow(Noise, CLOUD_AMOUNT - rainStrength*6)*1500;

    Noise = 1-1/(1+Noise);

    vec3 CloudColor = SKY_GROUND*0.3;
    vec2 CloudDir = get_cloud_lighting(CloudPos.xz).xz;
    float SunStrength = clamp(dot(-(to_player_pos(sunOrMoonPosN)).xz, CloudDir), -1, 1)+1;
    SunStrength = pow(SunStrength*(0.6-(rainStrength+nightStrength)*0.2), 3);
    CloudColor += SUN_DIRECT*SunStrength;

    Noise *= smoothstep(0.0, 0.2, CloudPos.y); // Reduce cloud amount further away

	return mix(SkyColor, CloudColor, Noise);
}

vec3 get_sky(vec3 ViewPosN, vec3 SunGlare) {
	float upDot = dot(ViewPosN, gbufferModelView[1].xyz)+0.1; //not much, what's up with you?

	vec3 MixedColor = mix(SKY_TOP, SKY_GROUND+SunGlare, fogify(max(upDot, 0.0), 0.05));
    return MixedColor;
}

float get_stars(vec3 PlayerPos) {
    vec3 StarCoord = PlayerPos / (PlayerPos.y + length(PlayerPos.xz));
    StarCoord.x += frameTimeCounter*0.001;
	const float ACTUAL_STAR_SIZE = STAR_SIZE * 512;
	StarCoord = floor(StarCoord * ACTUAL_STAR_SIZE) / ACTUAL_STAR_SIZE;

    float Visibility = smoothstep(StarCoord.y, -0.1, 0); // Smoothly fade out stars near the bottom of the sky
    #ifdef DIMENSION_OVERWORLD
    Visibility *= nightStrength;
    #endif
	return pow(random(StarCoord.xz), 1024) * Visibility * STAR_STRENGTH;
}

vec3 get_end_sky(vec3 pos, vec3 PlayerPos) {
    const vec3 SkyT = to_linear(vec3(END_SKY_T_R, END_SKY_T_G, END_SKY_T_B));
    const vec3 SkyG1 = to_linear(vec3(END_SKY_G_R, END_SKY_G_G, END_SKY_G_B));
    const vec3 SkyG2 = to_linear(vec3(END_SKY_G_RA, END_SKY_G_GA, END_SKY_G_BA));
	float upDot = dot(pos, gbufferModelView[1].xyz); //not much, what's up with you?
	vec2 RotPos1 = rotate(PlayerPos.xz, frameTimeCounter*0.02);
	vec2 RotPos2 = rotate(PlayerPos.xz, -frameTimeCounter*0.007);
	float Noise1 = fbm_fast(RotPos1*16, 2)*(1-abs(upDot));
	float Noise2 = fbm_fast(RotPos2*28, 2)*(1-abs(upDot));
    float Noise = (Noise1+Noise2);
    vec3 SkyG = SkyG1*Noise1+SkyG2*Noise2;
    vec3 Final = SkyT+SkyG*(fogify(upDot+0.2, 0.05));
    Final *= max(1-fogify(max(upDot+0.3, 0), 0.02), 0.01);
	return Final;
}

vec3 get_sky_main(vec3 ViewPosN, vec3 PlayerPosN, vec3 SunGlare) {
    #ifdef DIMENSION_OVERWORLD
    vec3 SkyColor = get_sky(ViewPosN, SunGlare);
	#elif defined DIMENSION_END
	vec3 SkyColor = get_end_sky(ViewPosN, PlayerPosN);
	#else
	vec3 fogColorL = to_linear(fogColor.rgb);
	float LightenFactor = pow(1-get_luminance(fogColorL), 4); // Makes all biomes roughly equally bright
	vec3 SkyColor = fogColorL*LightenFactor*3;
	#endif

    return SkyColor;
}

