#include "/lib/all_the_libs.glsl"

#include "/global/sky.glsl"
#include "/global/fog.glsl"
#include "/global/post/ssao.glsl"

// Sky, Clouds, etc

varying vec2 texcoord;

#if defined SSR && TAA_MODE == 0
/* DRAWBUFFERS:04 */
#else
/* DRAWBUFFERS:0 */
#endif

void main() {
	vec4 Color = texture2D(colortex0, texcoord);
	vec3 ScreenPos = vec3(gl_FragCoord.xy/resolution, gl_FragCoord.z);;
	float Depth = get_depth(ScreenPos.xy);
	vec3 ViewPos;
	#ifdef DISTANT_HORIZONS
	// Need to use dhProjectionInverse but only on DH terrain. Can't use #define DH_TERRAIN in this file
	if(texture2D(depthtex0, texcoord).x >= 1 && Depth < 1)
		ViewPos = to_view_pos_dh(vec3(texcoord, Depth));
	else
	#endif
		ViewPos = to_view_pos(vec3(texcoord, Depth));

	vec3 PlayerPos = mat3(gbufferModelViewInverse) * ViewPos;

	vec3 ViewPosN = normalize(ViewPos);
	vec3 PlayerPosN = normalize(PlayerPos);
	
	float Dist = distance(ViewPosN, sunPosN);
	vec3 SunGlare = get_sun_glare(Dist);
	vec3 SkyColor = get_sky_main(ViewPosN, PlayerPosN, SunGlare);

	if(Depth >= 1) {
		#ifdef DISTANT_HORIZONS
		Color.rgb = get_fog_main(PlayerPos, Color.rgb, gl_FragCoord.z, SkyColor);
		#endif
		#ifndef DIMENSION_OVERWORLD
		Color = vec4(0, 0, 0, 1);
		#elif defined ROUND_SUN
		SkyColor += round_sun(Dist);
		#endif
		Color.rgb += SkyColor;

		if(PlayerPos.y > 0) {
			#if defined DIMENSION_OVERWORLD || defined DIMENSION_END
			Color.xyz += get_stars(PlayerPos);
			#endif

			#if defined FANCY_CLOUDS && defined DIMENSION_OVERWORLD
				vec3 CloudPos = PlayerPosN;
				CloudPos.xz = PlayerPos.xz / (PlayerPos.y + length(PlayerPos.xz)/4);

				Color.rgb = get_clouds(CloudPos, Color.rgb, SunGlare);
			#endif
		}

	}
	
	else if(Depth >= 0.56) { // Skip hand & sky
		#ifdef SSAO
		vec3 Normal = decodeUnitVector(texture2D(colortex2, texcoord).rg * 2 - 1);
		Color.rgb = ssao(Color.rgb, Normal, ViewPos);
		#endif
		Color.rgb = get_fog_main(PlayerPos, Color.rgb, gl_FragCoord.z, SkyColor);
		#if defined SSR && TAA_MODE == 0
		gl_FragData[1] = Color;
		#endif
	}


	
	gl_FragData[0] = Color;
}

